# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input


@onnx_mm_export("v11.ConcatFromSequence")
def ConcatFromSequence(input_sequence, **kwargs):
  _inputs = []
  for i in (input_sequence, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ConcatFromSequence"]
  omm.op_counter["ConcatFromSequence"] += 1
  node = onnx.helper.make_node("ConcatFromSequence",
                               _inputs, [f"_t_ConcatFromSequence_{idx}"],
                               name=f"ConcatFromSequence_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceAt")
def SequenceAt(input_sequence, position, **kwargs):
  _inputs = []
  for i in (input_sequence, position):
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceAt"]
  omm.op_counter["SequenceAt"] += 1
  node = onnx.helper.make_node("SequenceAt",
                               _inputs, [f"_t_SequenceAt_{idx}"],
                               name=f"SequenceAt_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceInsert")
def SequenceInsert(input_sequence, tensor, position=None, **kwargs):
  _inputs = []
  for i in (input_sequence, tensor, position):
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceInsert"]
  omm.op_counter["SequenceInsert"] += 1
  node = onnx.helper.make_node("SequenceInsert",
                               _inputs, [f"_t_SequenceInsert_{idx}"],
                               name=f"SequenceInsert_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.GatherND")
def GatherND(data, indices, **kwargs):
  _inputs = []
  for i in (data, indices):
    _add_input(i, _inputs)

  idx = omm.op_counter["GatherND"]
  omm.op_counter["GatherND"] += 1
  node = onnx.helper.make_node("GatherND",
                               _inputs, [f"_t_GatherND_{idx}"],
                               name=f"GatherND_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ScatterND")
def ScatterND(data, indices, updates, **kwargs):
  _inputs = []
  for i in (data, indices, updates):
    _add_input(i, _inputs)

  idx = omm.op_counter["ScatterND"]
  omm.op_counter["ScatterND"] += 1
  node = onnx.helper.make_node("ScatterND",
                               _inputs, [f"_t_ScatterND_{idx}"],
                               name=f"ScatterND_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Det")
def Det(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Det"]
  omm.op_counter["Det"] += 1
  node = onnx.helper.make_node("Det",
                               _inputs, [f"_t_Det_{idx}"],
                               name=f"Det_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ScatterElements")
def ScatterElements(data, indices, updates, **kwargs):
  _inputs = []
  for i in (data, indices, updates):
    _add_input(i, _inputs)

  idx = omm.op_counter["ScatterElements"]
  omm.op_counter["ScatterElements"] += 1
  node = onnx.helper.make_node("ScatterElements",
                               _inputs, [f"_t_ScatterElements_{idx}"],
                               name=f"ScatterElements_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.GatherElements")
def GatherElements(data, indices, **kwargs):
  _inputs = []
  for i in (data, indices):
    _add_input(i, _inputs)

  idx = omm.op_counter["GatherElements"]
  omm.op_counter["GatherElements"] += 1
  node = onnx.helper.make_node("GatherElements",
                               _inputs, [f"_t_GatherElements_{idx}"],
                               name=f"GatherElements_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SplitToSequence")
def SplitToSequence(input, split=None, **kwargs):
  _inputs = []
  for i in (input, split):
    _add_input(i, _inputs)

  idx = omm.op_counter["SplitToSequence"]
  omm.op_counter["SplitToSequence"] += 1
  node = onnx.helper.make_node("SplitToSequence",
                               _inputs, [f"_t_SplitToSequence_{idx}"],
                               name=f"SplitToSequence_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.DynamicQuantizeLinear")
def DynamicQuantizeLinear(x, **kwargs):
  _inputs = []
  for i in (x, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["DynamicQuantizeLinear"]
  omm.op_counter["DynamicQuantizeLinear"] += 1
  node = onnx.helper.make_node("DynamicQuantizeLinear",
                               _inputs, [f"_t_DynamicQuantizeLinear_{idx}"],
                               name=f"DynamicQuantizeLinear_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Round")
def Round(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Round"]
  omm.op_counter["Round"] += 1
  node = onnx.helper.make_node("Round",
                               _inputs, [f"_t_Round_{idx}"],
                               name=f"Round_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.CumSum")
def CumSum(x, axis, **kwargs):
  _inputs = []
  for i in (x, axis):
    _add_input(i, _inputs)

  idx = omm.op_counter["CumSum"]
  omm.op_counter["CumSum"] += 1
  node = onnx.helper.make_node("CumSum",
                               _inputs, [f"_t_CumSum_{idx}"],
                               name=f"CumSum_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.BitShift")
def BitShift(X, Y, **kwargs):
  _inputs = []
  for i in (X, Y):
    _add_input(i, _inputs)

  idx = omm.op_counter["BitShift"]
  omm.op_counter["BitShift"] += 1
  node = onnx.helper.make_node("BitShift",
                               _inputs, [f"_t_BitShift_{idx}"],
                               name=f"BitShift_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.NonMaxSuppression")
def NonMaxSuppression(boxes, scores, max_output_boxes_per_class=None, iou_threshold=None, score_threshold=None, **kwargs):
  _inputs = []
  for i in (boxes, scores, max_output_boxes_per_class, iou_threshold, score_threshold):
    _add_input(i, _inputs)

  idx = omm.op_counter["NonMaxSuppression"]
  omm.op_counter["NonMaxSuppression"] += 1
  node = onnx.helper.make_node("NonMaxSuppression",
                               _inputs, [f"_t_NonMaxSuppression_{idx}"],
                               name=f"NonMaxSuppression_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Range")
def Range(start, limit, delta, **kwargs):
  _inputs = []
  for i in (start, limit, delta):
    _add_input(i, _inputs)

  idx = omm.op_counter["Range"]
  omm.op_counter["Range"] += 1
  node = onnx.helper.make_node("Range",
                               _inputs, [f"_t_Range_{idx}"],
                               name=f"Range_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceErase")
def SequenceErase(input_sequence, position=None, **kwargs):
  _inputs = []
  for i in (input_sequence, position):
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceErase"]
  omm.op_counter["SequenceErase"] += 1
  node = onnx.helper.make_node("SequenceErase",
                               _inputs, [f"_t_SequenceErase_{idx}"],
                               name=f"SequenceErase_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Scatter")
def Scatter(data, indices, updates, **kwargs):
  _inputs = []
  for i in (data, indices, updates):
    _add_input(i, _inputs)

  idx = omm.op_counter["Scatter"]
  omm.op_counter["Scatter"] += 1
  node = onnx.helper.make_node("Scatter",
                               _inputs, [f"_t_Scatter_{idx}"],
                               name=f"Scatter_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.OneHot")
def OneHot(indices, depth, values, **kwargs):
  _inputs = []
  for i in (indices, depth, values):
    _add_input(i, _inputs)

  idx = omm.op_counter["OneHot"]
  omm.op_counter["OneHot"] += 1
  node = onnx.helper.make_node("OneHot",
                               _inputs, [f"_t_OneHot_{idx}"],
                               name=f"OneHot_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.MaxUnpool")
def MaxUnpool(X, I, output_shape=None, **kwargs):
  _inputs = []
  for i in (X, I, output_shape):
    _add_input(i, _inputs)

  idx = omm.op_counter["MaxUnpool"]
  omm.op_counter["MaxUnpool"] += 1
  node = onnx.helper.make_node("MaxUnpool",
                               _inputs, [f"_t_MaxUnpool_{idx}"],
                               name=f"MaxUnpool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Compress")
def Compress(input, condition, **kwargs):
  _inputs = []
  for i in (input, condition):
    _add_input(i, _inputs)

  idx = omm.op_counter["Compress"]
  omm.op_counter["Compress"] += 1
  node = onnx.helper.make_node("Compress",
                               _inputs, [f"_t_Compress_{idx}"],
                               name=f"Compress_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Scan")
def Scan(initial_state_and_scan_inputs, **kwargs):
  _inputs = []
  for i in (initial_state_and_scan_inputs, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Scan"]
  omm.op_counter["Scan"] += 1
  node = onnx.helper.make_node("Scan",
                               _inputs, [f"_t_Scan_{idx}"],
                               name=f"Scan_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceSumSquare")
def ReduceSumSquare(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceSumSquare"]
  omm.op_counter["ReduceSumSquare"] += 1
  node = onnx.helper.make_node("ReduceSumSquare",
                               _inputs, [f"_t_ReduceSumSquare_{idx}"],
                               name=f"ReduceSumSquare_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Squeeze")
def Squeeze(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Squeeze"]
  omm.op_counter["Squeeze"] += 1
  node = onnx.helper.make_node("Squeeze",
                               _inputs, [f"_t_Squeeze_{idx}"],
                               name=f"Squeeze_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceLogSum")
def ReduceLogSum(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceLogSum"]
  omm.op_counter["ReduceLogSum"] += 1
  node = onnx.helper.make_node("ReduceLogSum",
                               _inputs, [f"_t_ReduceLogSum_{idx}"],
                               name=f"ReduceLogSum_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Split")
def Split(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Split"]
  omm.op_counter["Split"] += 1
  node = onnx.helper.make_node("Split",
                               _inputs, [f"_t_Split_{idx}_{i}" for i in range(len(kwargs["split"]))],
                               name=f"Split_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Softmax")
def Softmax(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Softmax"]
  omm.op_counter["Softmax"] += 1
  node = onnx.helper.make_node("Softmax",
                               _inputs, [f"_t_Softmax_{idx}"],
                               name=f"Softmax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceLength")
def SequenceLength(input_sequence, **kwargs):
  _inputs = []
  for i in (input_sequence, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceLength"]
  omm.op_counter["SequenceLength"] += 1
  node = onnx.helper.make_node("SequenceLength",
                               _inputs, [f"_t_SequenceLength_{idx}"],
                               name=f"SequenceLength_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceMin")
def ReduceMin(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceMin"]
  omm.op_counter["ReduceMin"] += 1
  node = onnx.helper.make_node("ReduceMin",
                               _inputs, [f"_t_ReduceMin_{idx}"],
                               name=f"ReduceMin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceL1")
def ReduceL1(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceL1"]
  omm.op_counter["ReduceL1"] += 1
  node = onnx.helper.make_node("ReduceL1",
                               _inputs, [f"_t_ReduceL1_{idx}"],
                               name=f"ReduceL1_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Pad")
def Pad(data, pads, constant_value=None, **kwargs):
  _inputs = []
  for i in (data, pads, constant_value):
    _add_input(i, _inputs)

  idx = omm.op_counter["Pad"]
  omm.op_counter["Pad"] += 1
  node = onnx.helper.make_node("Pad",
                               _inputs, [f"_t_Pad_{idx}"],
                               name=f"Pad_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Slice")
def Slice(data, starts, ends, axes=None, steps=None, **kwargs):
  _inputs = []
  for i in (data, starts, ends, axes, steps):
    _add_input(i, _inputs)

  idx = omm.op_counter["Slice"]
  omm.op_counter["Slice"] += 1
  node = onnx.helper.make_node("Slice",
                               _inputs, [f"_t_Slice_{idx}"],
                               name=f"Slice_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceLogSumExp")
def ReduceLogSumExp(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceLogSumExp"]
  omm.op_counter["ReduceLogSumExp"] += 1
  node = onnx.helper.make_node("ReduceLogSumExp",
                               _inputs, [f"_t_ReduceLogSumExp_{idx}"],
                               name=f"ReduceLogSumExp_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceL2")
def ReduceL2(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceL2"]
  omm.op_counter["ReduceL2"] += 1
  node = onnx.helper.make_node("ReduceL2",
                               _inputs, [f"_t_ReduceL2_{idx}"],
                               name=f"ReduceL2_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Flatten")
def Flatten(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Flatten"]
  omm.op_counter["Flatten"] += 1
  node = onnx.helper.make_node("Flatten",
                               _inputs, [f"_t_Flatten_{idx}"],
                               name=f"Flatten_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Conv")
def Conv(X, W, B=None, **kwargs):
  _inputs = []
  for i in (X, W, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Conv"]
  omm.op_counter["Conv"] += 1
  node = onnx.helper.make_node("Conv",
                               _inputs, [f"_t_Conv_{idx}"],
                               name=f"Conv_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.LpPool")
def LpPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["LpPool"]
  omm.op_counter["LpPool"] += 1
  node = onnx.helper.make_node("LpPool",
                               _inputs, [f"_t_LpPool_{idx}"],
                               name=f"LpPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceMax")
def ReduceMax(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceMax"]
  omm.op_counter["ReduceMax"] += 1
  node = onnx.helper.make_node("ReduceMax",
                               _inputs, [f"_t_ReduceMax_{idx}"],
                               name=f"ReduceMax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Loop")
def Loop(M, cond, v_initial=None, **kwargs):
  _inputs = []
  for i in (M, cond, v_initial):
    _add_input(i, _inputs)

  idx = omm.op_counter["Loop"]
  omm.op_counter["Loop"] += 1
  node = onnx.helper.make_node("Loop",
                               _inputs, [f"_t_Loop_{idx}"],
                               name=f"Loop_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Unsqueeze")
def Unsqueeze(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Unsqueeze"]
  omm.op_counter["Unsqueeze"] += 1
  node = onnx.helper.make_node("Unsqueeze",
                               _inputs, [f"_t_Unsqueeze_{idx}"],
                               name=f"Unsqueeze_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.TopK")
def TopK(X, K, **kwargs):
  _inputs = []
  for i in (X, K):
    _add_input(i, _inputs)

  idx = omm.op_counter["TopK"]
  omm.op_counter["TopK"] += 1
  node = onnx.helper.make_node("TopK",
                               _inputs, [f"_t_TopK_{idx}"],
                               name=f"TopK_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ArgMax")
def ArgMax(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ArgMax"]
  omm.op_counter["ArgMax"] += 1
  node = onnx.helper.make_node("ArgMax",
                               _inputs, [f"_t_ArgMax_{idx}"],
                               name=f"ArgMax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceEmpty")
def SequenceEmpty(**kwargs):
  _inputs = []
  for i in ():
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceEmpty"]
  omm.op_counter["SequenceEmpty"] += 1
  node = onnx.helper.make_node("SequenceEmpty",
                               _inputs, [f"_t_SequenceEmpty_{idx}"],
                               name=f"SequenceEmpty_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Concat")
def Concat(inputs, **kwargs):
  _inputs = []
  for i in (inputs, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Concat"]
  omm.op_counter["Concat"] += 1
  node = onnx.helper.make_node("Concat",
                               _inputs, [f"_t_Concat_{idx}"],
                               name=f"Concat_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.If")
def If(cond, **kwargs):
  _inputs = []
  for i in (cond, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["If"]
  omm.op_counter["If"] += 1
  node = onnx.helper.make_node("If",
                               _inputs, [f"_t_If_{idx}"],
                               name=f"If_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Clip")
def Clip(input, min=None, max=None, **kwargs):
  _inputs = []
  for i in (input, min, max):
    _add_input(i, _inputs)

  idx = omm.op_counter["Clip"]
  omm.op_counter["Clip"] += 1
  node = onnx.helper.make_node("Clip",
                               _inputs, [f"_t_Clip_{idx}"],
                               name=f"Clip_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceProd")
def ReduceProd(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceProd"]
  omm.op_counter["ReduceProd"] += 1
  node = onnx.helper.make_node("ReduceProd",
                               _inputs, [f"_t_ReduceProd_{idx}"],
                               name=f"ReduceProd_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Gather")
def Gather(data, indices, **kwargs):
  _inputs = []
  for i in (data, indices):
    _add_input(i, _inputs)

  idx = omm.op_counter["Gather"]
  omm.op_counter["Gather"] += 1
  node = onnx.helper.make_node("Gather",
                               _inputs, [f"_t_Gather_{idx}"],
                               name=f"Gather_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Resize")
def Resize(X, roi, scales, sizes=None, **kwargs):
  _inputs = []
  for i in (X, roi, scales, sizes):
    _add_input(i, _inputs)

  idx = omm.op_counter["Resize"]
  omm.op_counter["Resize"] += 1
  node = onnx.helper.make_node("Resize",
                               _inputs, [f"_t_Resize_{idx}"],
                               name=f"Resize_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.SequenceConstruct")
def SequenceConstruct(inputs, **kwargs):
  _inputs = []
  for i in (inputs, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["SequenceConstruct"]
  omm.op_counter["SequenceConstruct"] += 1
  node = onnx.helper.make_node("SequenceConstruct",
                               _inputs, [f"_t_SequenceConstruct_{idx}"],
                               name=f"SequenceConstruct_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Unique")
def Unique(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Unique"]
  omm.op_counter["Unique"] += 1
  node = onnx.helper.make_node("Unique",
                               _inputs, [f"_t_Unique_{idx}"],
                               name=f"Unique_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ArgMin")
def ArgMin(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ArgMin"]
  omm.op_counter["ArgMin"] += 1
  node = onnx.helper.make_node("ArgMin",
                               _inputs, [f"_t_ArgMin_{idx}"],
                               name=f"ArgMin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Constant")
def Constant(**kwargs):
  _inputs = []
  for i in ():
    _add_input(i, _inputs)

  idx = omm.op_counter["Constant"]
  omm.op_counter["Constant"] += 1
  node = onnx.helper.make_node("Constant",
                               _inputs, [f"_t_Constant_{idx}"],
                               name=f"Constant_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Equal")
def Equal(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Equal"]
  omm.op_counter["Equal"] += 1
  node = onnx.helper.make_node("Equal",
                               _inputs, [f"_t_Equal_{idx}"],
                               name=f"Equal_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceSum")
def ReduceSum(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceSum"]
  omm.op_counter["ReduceSum"] += 1
  node = onnx.helper.make_node("ReduceSum",
                               _inputs, [f"_t_ReduceSum_{idx}"],
                               name=f"ReduceSum_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.MaxPool")
def MaxPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["MaxPool"]
  omm.op_counter["MaxPool"] += 1
  node = onnx.helper.make_node("MaxPool",
                               _inputs, [f"_t_MaxPool_{idx}"],
                               name=f"MaxPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ReduceMean")
def ReduceMean(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceMean"]
  omm.op_counter["ReduceMean"] += 1
  node = onnx.helper.make_node("ReduceMean",
                               _inputs, [f"_t_ReduceMean_{idx}"],
                               name=f"ReduceMean_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.DepthToSpace")
def DepthToSpace(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["DepthToSpace"]
  omm.op_counter["DepthToSpace"] += 1
  node = onnx.helper.make_node("DepthToSpace",
                               _inputs, [f"_t_DepthToSpace_{idx}"],
                               name=f"DepthToSpace_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.LogSoftmax")
def LogSoftmax(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["LogSoftmax"]
  omm.op_counter["LogSoftmax"] += 1
  node = onnx.helper.make_node("LogSoftmax",
                               _inputs, [f"_t_LogSoftmax_{idx}"],
                               name=f"LogSoftmax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.AveragePool")
def AveragePool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["AveragePool"]
  omm.op_counter["AveragePool"] += 1
  node = onnx.helper.make_node("AveragePool",
                               _inputs, [f"_t_AveragePool_{idx}"],
                               name=f"AveragePool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Hardmax")
def Hardmax(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Hardmax"]
  omm.op_counter["Hardmax"] += 1
  node = onnx.helper.make_node("Hardmax",
                               _inputs, [f"_t_Hardmax_{idx}"],
                               name=f"Hardmax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.ConvTranspose")
def ConvTranspose(X, W, B=None, **kwargs):
  _inputs = []
  for i in (X, W, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["ConvTranspose"]
  omm.op_counter["ConvTranspose"] += 1
  node = onnx.helper.make_node("ConvTranspose",
                               _inputs, [f"_t_ConvTranspose_{idx}"],
                               name=f"ConvTranspose_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v11.Gemm")
def Gemm(A, B, C=None, **kwargs):
  _inputs = []
  for i in (A, B, C):
    _add_input(i, _inputs)

  idx = omm.op_counter["Gemm"]
  omm.op_counter["Gemm"] += 1
  node = onnx.helper.make_node("Gemm",
                               _inputs, [f"_t_Gemm_{idx}"],
                               name=f"Gemm_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node



# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input


@onnx_mm_export("v2.LabelEncoder")
def LabelEncoder(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["LabelEncoder"]
  omm.op_counter["LabelEncoder"] += 1
  node = onnx.helper.make_node("LabelEncoder",
                               _inputs, [f"_t_LabelEncoder_{idx}"],
                               name=f"LabelEncoder_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v2.Split")
def Split(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Split"]
  omm.op_counter["Split"] += 1
  node = onnx.helper.make_node("Split",
                               _inputs, [f"_t_Split_{idx}_{i}" for i in range(len(kwargs["split"]))],
                               name=f"Split_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v2.Pad")
def Pad(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Pad"]
  omm.op_counter["Pad"] += 1
  node = onnx.helper.make_node("Pad",
                               _inputs, [f"_t_Pad_{idx}"],
                               name=f"Pad_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v2.LpPool")
def LpPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["LpPool"]
  omm.op_counter["LpPool"] += 1
  node = onnx.helper.make_node("LpPool",
                               _inputs, [f"_t_LpPool_{idx}"],
                               name=f"LpPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v2.GlobalLpPool")
def GlobalLpPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["GlobalLpPool"]
  omm.op_counter["GlobalLpPool"] += 1
  node = onnx.helper.make_node("GlobalLpPool",
                               _inputs, [f"_t_GlobalLpPool_{idx}"],
                               name=f"GlobalLpPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node



import onnx
import torch

from onnx_pytorch.op_code_generators import OpCodeGenerator


class MaxPoolOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(MaxPoolOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    attr_value_dict = self.get_attr_value_dict(node)
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)

    d = len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2
    assert (d in (1, 2, 3))

    params_str = self.gen_params_str(
        dilation=attr_value_dict.get("dilations", 1),
        kernel_size=attr_value_dict["kernel_shape"][:].__repr__(),
        ceil_mode=bool(attr_value_dict["ceil_mode"]),
        stride=attr_value_dict.get("strides", 1),
        return_indices=(len(node.output) == 2))

    nn_name = f"MaxPool{d}d"
    init_str, forward_str = [], []
    init_str.append(f"self.{node.name} = nn.{nn_name}(**{{{params_str}}})")
    if "pads" in attr_value_dict:
      padding = []
      for i in range(d):
        padding.insert(0, attr_value_dict['pads'][i + d])
        padding.insert(0, attr_value_dict['pads'][i])
      forward_str.append(
          f"{inputs_str[0]} = torch.nn.functional.pad({inputs_str[0]}, {padding.__repr__()}, value=float('-inf'))"
      )
    forward_str.append(f"{outputs_str[0]} = self.{node.name}({inputs_str[0]})")

    return {"init": init_str, "forward": forward_str}



# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input


@onnx_mm_export("v7.Multinomial")
def Multinomial(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Multinomial"]
  omm.op_counter["Multinomial"] += 1
  node = onnx.helper.make_node("Multinomial",
                               _inputs, [f"_t_Multinomial_{idx}"],
                               name=f"Multinomial_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Asin")
def Asin(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Asin"]
  omm.op_counter["Asin"] += 1
  node = onnx.helper.make_node("Asin",
                               _inputs, [f"_t_Asin_{idx}"],
                               name=f"Asin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Xor")
def Xor(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Xor"]
  omm.op_counter["Xor"] += 1
  node = onnx.helper.make_node("Xor",
                               _inputs, [f"_t_Xor_{idx}"],
                               name=f"Xor_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Upsample")
def Upsample(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Upsample"]
  omm.op_counter["Upsample"] += 1
  node = onnx.helper.make_node("Upsample",
                               _inputs, [f"_t_Upsample_{idx}"],
                               name=f"Upsample_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.And")
def And(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["And"]
  omm.op_counter["And"] += 1
  node = onnx.helper.make_node("And",
                               _inputs, [f"_t_And_{idx}"],
                               name=f"And_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Sub")
def Sub(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Sub"]
  omm.op_counter["Sub"] += 1
  node = onnx.helper.make_node("Sub",
                               _inputs, [f"_t_Sub_{idx}"],
                               name=f"Sub_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Cos")
def Cos(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Cos"]
  omm.op_counter["Cos"] += 1
  node = onnx.helper.make_node("Cos",
                               _inputs, [f"_t_Cos_{idx}"],
                               name=f"Cos_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Mul")
def Mul(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Mul"]
  omm.op_counter["Mul"] += 1
  node = onnx.helper.make_node("Mul",
                               _inputs, [f"_t_Mul_{idx}"],
                               name=f"Mul_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Sin")
def Sin(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Sin"]
  omm.op_counter["Sin"] += 1
  node = onnx.helper.make_node("Sin",
                               _inputs, [f"_t_Sin_{idx}"],
                               name=f"Sin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.RNN")
def RNN(X, W, R, B=None, sequence_lens=None, initial_h=None, **kwargs):
  _inputs = []
  for i in (X, W, R, B, sequence_lens, initial_h):
    _add_input(i, _inputs)

  idx = omm.op_counter["RNN"]
  omm.op_counter["RNN"] += 1
  node = onnx.helper.make_node("RNN",
                               _inputs, [f"_t_RNN_{idx}"],
                               name=f"RNN_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Greater")
def Greater(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Greater"]
  omm.op_counter["Greater"] += 1
  node = onnx.helper.make_node("Greater",
                               _inputs, [f"_t_Greater_{idx}"],
                               name=f"Greater_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Or")
def Or(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Or"]
  omm.op_counter["Or"] += 1
  node = onnx.helper.make_node("Or",
                               _inputs, [f"_t_Or_{idx}"],
                               name=f"Or_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.BatchNormalization")
def BatchNormalization(X, scale, B, mean, var, **kwargs):
  _inputs = []
  for i in (X, scale, B, mean, var):
    _add_input(i, _inputs)

  idx = omm.op_counter["BatchNormalization"]
  omm.op_counter["BatchNormalization"] += 1
  node = onnx.helper.make_node("BatchNormalization",
                               _inputs, [f"_t_BatchNormalization_{idx}"],
                               name=f"BatchNormalization_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.LSTM")
def LSTM(X, W, R, B=None, sequence_lens=None, initial_h=None, initial_c=None, P=None, **kwargs):
  _inputs = []
  for i in (X, W, R, B, sequence_lens, initial_h, initial_c, P):
    _add_input(i, _inputs)

  idx = omm.op_counter["LSTM"]
  omm.op_counter["LSTM"] += 1
  node = onnx.helper.make_node("LSTM",
                               _inputs, [f"_t_LSTM_{idx}"],
                               name=f"LSTM_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Acos")
def Acos(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Acos"]
  omm.op_counter["Acos"] += 1
  node = onnx.helper.make_node("Acos",
                               _inputs, [f"_t_Acos_{idx}"],
                               name=f"Acos_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.PRelu")
def PRelu(X, slope, **kwargs):
  _inputs = []
  for i in (X, slope):
    _add_input(i, _inputs)

  idx = omm.op_counter["PRelu"]
  omm.op_counter["PRelu"] += 1
  node = onnx.helper.make_node("PRelu",
                               _inputs, [f"_t_PRelu_{idx}"],
                               name=f"PRelu_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.GRU")
def GRU(X, W, R, B=None, sequence_lens=None, initial_h=None, **kwargs):
  _inputs = []
  for i in (X, W, R, B, sequence_lens, initial_h):
    _add_input(i, _inputs)

  idx = omm.op_counter["GRU"]
  omm.op_counter["GRU"] += 1
  node = onnx.helper.make_node("GRU",
                               _inputs, [f"_t_GRU_{idx}"],
                               name=f"GRU_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Tan")
def Tan(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Tan"]
  omm.op_counter["Tan"] += 1
  node = onnx.helper.make_node("Tan",
                               _inputs, [f"_t_Tan_{idx}"],
                               name=f"Tan_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Add")
def Add(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Add"]
  omm.op_counter["Add"] += 1
  node = onnx.helper.make_node("Add",
                               _inputs, [f"_t_Add_{idx}"],
                               name=f"Add_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Equal")
def Equal(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Equal"]
  omm.op_counter["Equal"] += 1
  node = onnx.helper.make_node("Equal",
                               _inputs, [f"_t_Equal_{idx}"],
                               name=f"Equal_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Pow")
def Pow(X, Y, **kwargs):
  _inputs = []
  for i in (X, Y):
    _add_input(i, _inputs)

  idx = omm.op_counter["Pow"]
  omm.op_counter["Pow"] += 1
  node = onnx.helper.make_node("Pow",
                               _inputs, [f"_t_Pow_{idx}"],
                               name=f"Pow_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Div")
def Div(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Div"]
  omm.op_counter["Div"] += 1
  node = onnx.helper.make_node("Div",
                               _inputs, [f"_t_Div_{idx}"],
                               name=f"Div_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Less")
def Less(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["Less"]
  omm.op_counter["Less"] += 1
  node = onnx.helper.make_node("Less",
                               _inputs, [f"_t_Less_{idx}"],
                               name=f"Less_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Dropout")
def Dropout(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Dropout"]
  omm.op_counter["Dropout"] += 1
  node = onnx.helper.make_node("Dropout",
                               _inputs, [f"_t_Dropout_{idx}"],
                               name=f"Dropout_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Atan")
def Atan(input, **kwargs):
  _inputs = []
  for i in (input, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Atan"]
  omm.op_counter["Atan"] += 1
  node = onnx.helper.make_node("Atan",
                               _inputs, [f"_t_Atan_{idx}"],
                               name=f"Atan_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.AveragePool")
def AveragePool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["AveragePool"]
  omm.op_counter["AveragePool"] += 1
  node = onnx.helper.make_node("AveragePool",
                               _inputs, [f"_t_AveragePool_{idx}"],
                               name=f"AveragePool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v7.Gemm")
def Gemm(A, B, C, **kwargs):
  _inputs = []
  for i in (A, B, C):
    _add_input(i, _inputs)

  idx = omm.op_counter["Gemm"]
  omm.op_counter["Gemm"] += 1
  node = onnx.helper.make_node("Gemm",
                               _inputs, [f"_t_Gemm_{idx}"],
                               name=f"Gemm_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node



import onnx
import onnx.numpy_helper
import torch

from onnx_pytorch.op_code_generators import OpCodeGenerator


class BatchNormalizationOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(BatchNormalizationOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    attr_value_dict = self.get_attr_value_dict(node)
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)

    d = len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2
    assert (d in (1, 2, 3))

    nn_name = f"BatchNorm{d}d"

    params_str = self.gen_params_str(num_features=onnx.numpy_helper.to_array(
        initializers[node.input[1]]).shape[0],
                                     eps=attr_value_dict["epsilon"],
                                     momentum=attr_value_dict["momentum"])

    init_str, forward_str = [], []
    init_str.append(f"self.{node.name} = nn.{nn_name}(**{{{params_str}}})")
    init_str.append(
        f"self.{node.name}.weight.data = self.__variables[\"{node.input[1]}\"]")
    init_str.append(
        f"self.{node.name}.bias.data = self.__variables[\"{node.input[2]}\"]")
    init_str.append(
        f"self.{node.name}.running_mean.data = self.__variables[\"{node.input[3]}\"]"
    )
    init_str.append(
        f"self.{node.name}.running_var.data = self.__variables[\"{node.input[4]}\"]"
    )
    forward_str.append(f"{outputs_str[0]} = self.{node.name}({inputs_str[0]})")
    return {"init": init_str, "forward": forward_str}



# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input


@onnx_mm_export("v12.LessOrEqual")
def LessOrEqual(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["LessOrEqual"]
  omm.op_counter["LessOrEqual"] += 1
  node = onnx.helper.make_node("LessOrEqual",
                               _inputs, [f"_t_LessOrEqual_{idx}"],
                               name=f"LessOrEqual_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Celu")
def Celu(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Celu"]
  omm.op_counter["Celu"] += 1
  node = onnx.helper.make_node("Celu",
                               _inputs, [f"_t_Celu_{idx}"],
                               name=f"Celu_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.GatherND")
def GatherND(data, indices, **kwargs):
  _inputs = []
  for i in (data, indices):
    _add_input(i, _inputs)

  idx = omm.op_counter["GatherND"]
  omm.op_counter["GatherND"] += 1
  node = onnx.helper.make_node("GatherND",
                               _inputs, [f"_t_GatherND_{idx}"],
                               name=f"GatherND_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Einsum")
def Einsum(Inputs, **kwargs):
  _inputs = []
  for i in (Inputs, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Einsum"]
  omm.op_counter["Einsum"] += 1
  node = onnx.helper.make_node("Einsum",
                               _inputs, [f"_t_Einsum_{idx}"],
                               name=f"Einsum_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.GreaterOrEqual")
def GreaterOrEqual(A, B, **kwargs):
  _inputs = []
  for i in (A, B):
    _add_input(i, _inputs)

  idx = omm.op_counter["GreaterOrEqual"]
  omm.op_counter["GreaterOrEqual"] += 1
  node = onnx.helper.make_node("GreaterOrEqual",
                               _inputs, [f"_t_GreaterOrEqual_{idx}"],
                               name=f"GreaterOrEqual_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Max")
def Max(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Max"]
  omm.op_counter["Max"] += 1
  node = onnx.helper.make_node("Max",
                               _inputs, [f"_t_Max_{idx}"],
                               name=f"Max_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.NegativeLogLikelihoodLoss")
def NegativeLogLikelihoodLoss(input, target, weight=None, **kwargs):
  _inputs = []
  for i in (input, target, weight):
    _add_input(i, _inputs)

  idx = omm.op_counter["NegativeLogLikelihoodLoss"]
  omm.op_counter["NegativeLogLikelihoodLoss"] += 1
  node = onnx.helper.make_node("NegativeLogLikelihoodLoss",
                               _inputs, [f"_t_NegativeLogLikelihoodLoss_{idx}"],
                               name=f"NegativeLogLikelihoodLoss_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.ReduceMin")
def ReduceMin(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceMin"]
  omm.op_counter["ReduceMin"] += 1
  node = onnx.helper.make_node("ReduceMin",
                               _inputs, [f"_t_ReduceMin_{idx}"],
                               name=f"ReduceMin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.ReduceMax")
def ReduceMax(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ReduceMax"]
  omm.op_counter["ReduceMax"] += 1
  node = onnx.helper.make_node("ReduceMax",
                               _inputs, [f"_t_ReduceMax_{idx}"],
                               name=f"ReduceMax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.ArgMax")
def ArgMax(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ArgMax"]
  omm.op_counter["ArgMax"] += 1
  node = onnx.helper.make_node("ArgMax",
                               _inputs, [f"_t_ArgMax_{idx}"],
                               name=f"ArgMax_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.SoftmaxCrossEntropyLoss")
def SoftmaxCrossEntropyLoss(scores, labels, weights=None, **kwargs):
  _inputs = []
  for i in (scores, labels, weights):
    _add_input(i, _inputs)

  idx = omm.op_counter["SoftmaxCrossEntropyLoss"]
  omm.op_counter["SoftmaxCrossEntropyLoss"] += 1
  node = onnx.helper.make_node("SoftmaxCrossEntropyLoss",
                               _inputs, [f"_t_SoftmaxCrossEntropyLoss_{idx}"],
                               name=f"SoftmaxCrossEntropyLoss_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Clip")
def Clip(input, min=None, max=None, **kwargs):
  _inputs = []
  for i in (input, min, max):
    _add_input(i, _inputs)

  idx = omm.op_counter["Clip"]
  omm.op_counter["Clip"] += 1
  node = onnx.helper.make_node("Clip",
                               _inputs, [f"_t_Clip_{idx}"],
                               name=f"Clip_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.ArgMin")
def ArgMin(data, **kwargs):
  _inputs = []
  for i in (data, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["ArgMin"]
  omm.op_counter["ArgMin"] += 1
  node = onnx.helper.make_node("ArgMin",
                               _inputs, [f"_t_ArgMin_{idx}"],
                               name=f"ArgMin_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Constant")
def Constant(**kwargs):
  _inputs = []
  for i in ():
    _add_input(i, _inputs)

  idx = omm.op_counter["Constant"]
  omm.op_counter["Constant"] += 1
  node = onnx.helper.make_node("Constant",
                               _inputs, [f"_t_Constant_{idx}"],
                               name=f"Constant_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Pow")
def Pow(X, Y, **kwargs):
  _inputs = []
  for i in (X, Y):
    _add_input(i, _inputs)

  idx = omm.op_counter["Pow"]
  omm.op_counter["Pow"] += 1
  node = onnx.helper.make_node("Pow",
                               _inputs, [f"_t_Pow_{idx}"],
                               name=f"Pow_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.MaxPool")
def MaxPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["MaxPool"]
  omm.op_counter["MaxPool"] += 1
  node = onnx.helper.make_node("MaxPool",
                               _inputs, [f"_t_MaxPool_{idx}"],
                               name=f"MaxPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Min")
def Min(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Min"]
  omm.op_counter["Min"] += 1
  node = onnx.helper.make_node("Min",
                               _inputs, [f"_t_Min_{idx}"],
                               name=f"Min_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v12.Dropout")
def Dropout(data, ratio=None, training_mode=None, **kwargs):
  _inputs = []
  for i in (data, ratio, training_mode):
    _add_input(i, _inputs)

  idx = omm.op_counter["Dropout"]
  omm.op_counter["Dropout"] += 1
  node = onnx.helper.make_node("Dropout",
                               _inputs, [f"_t_Dropout_{idx}"],
                               name=f"Dropout_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node



# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input


@onnx_mm_export("v8.Scan")
def Scan(sequence_lens, initial_state_and_scan_inputs, **kwargs):
  _inputs = []
  for i in (sequence_lens, initial_state_and_scan_inputs):
    _add_input(i, _inputs)

  idx = omm.op_counter["Scan"]
  omm.op_counter["Scan"] += 1
  node = onnx.helper.make_node("Scan",
                               _inputs, [f"_t_Scan_{idx}"],
                               name=f"Scan_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.Expand")
def Expand(input, shape, **kwargs):
  _inputs = []
  for i in (input, shape):
    _add_input(i, _inputs)

  idx = omm.op_counter["Expand"]
  omm.op_counter["Expand"] += 1
  node = onnx.helper.make_node("Expand",
                               _inputs, [f"_t_Expand_{idx}"],
                               name=f"Expand_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.Max")
def Max(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Max"]
  omm.op_counter["Max"] += 1
  node = onnx.helper.make_node("Max",
                               _inputs, [f"_t_Max_{idx}"],
                               name=f"Max_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.Sum")
def Sum(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Sum"]
  omm.op_counter["Sum"] += 1
  node = onnx.helper.make_node("Sum",
                               _inputs, [f"_t_Sum_{idx}"],
                               name=f"Sum_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.Mean")
def Mean(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Mean"]
  omm.op_counter["Mean"] += 1
  node = onnx.helper.make_node("Mean",
                               _inputs, [f"_t_Mean_{idx}"],
                               name=f"Mean_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.MaxPool")
def MaxPool(X, **kwargs):
  _inputs = []
  for i in (X, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["MaxPool"]
  omm.op_counter["MaxPool"] += 1
  node = onnx.helper.make_node("MaxPool",
                               _inputs, [f"_t_MaxPool_{idx}"],
                               name=f"MaxPool_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node


@onnx_mm_export("v8.Min")
def Min(data_0, **kwargs):
  _inputs = []
  for i in (data_0, ):
    _add_input(i, _inputs)

  idx = omm.op_counter["Min"]
  omm.op_counter["Min"] += 1
  node = onnx.helper.make_node("Min",
                               _inputs, [f"_t_Min_{idx}"],
                               name=f"Min_{idx}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node



import collections
import os
import shutil

import onnx

TENSOR_PREFIX = "_t_"
HEADER = f'''# Autogenerated by onnx-model-maker. Don't modify it manually.

import onnx
import onnx.helper
import onnx.numpy_helper
from onnx_model_maker import omm
from onnx_model_maker import onnx_mm_export
from onnx_model_maker.ops.op_helper import _add_input
'''

OP_HELPER_PY = f'''from uuid import uuid4

import numpy
import onnx

from onnx_model_maker import omm


def _add_input(target, inputs):
  if target is None:
    return
  if type(target) == numpy.ndarray:
    t = onnx.numpy_helper.from_array(target, f"{TENSOR_PREFIX}{{uuid4().hex[:4]}}")
    omm.model.graph.initializer.append(t)
    inputs.append(t.name)
  elif type(target) == str:
    inputs.append(target)
  elif type(target) == list and all([type(i) == str for i in target]):
    inputs.extend(target)
  elif type(target) == onnx.NodeProto:
    inputs.append(target.output[0])
'''

INIT_PY = f'''import glob
import importlib
import os
import sys

import onnx
import numpy

from onnx_model_maker import mod_name
from onnx_model_maker import omm
from onnx_model_maker import OPSET_VER


modules = glob.glob(os.path.join(os.path.dirname(__file__), "op_ver_*.py"))
for m in modules:
  spec = importlib.util.spec_from_file_location(os.path.basename(m)[:-3], m)
  spec.loader.exec_module(importlib.util.module_from_spec(spec))


def Input(*args):
  inputs = []
  for i, a in enumerate(args):
    t = onnx.numpy_helper.from_array(a)
    vi = onnx.helper.make_tensor_value_info(f"{TENSOR_PREFIX}Input_{{i}}",
                                            t.data_type, t.dims)
    omm.model.graph.input.append(vi)
    inputs.append(vi.name)
  return inputs


def Output(*args):
  for i, a in enumerate(args):
    if type(a) == numpy.ndarray:
      t = onnx.numpy_helper.from_array(a)
      vi = onnx.helper.make_tensor_value_info(f"{TENSOR_PREFIX}Output_{{i}}", t.data_type,
                                              t.dims)
      omm.model.graph.output.append(vi)
    elif type(a) == str:
      vi = onnx.helper.make_empty_tensor_value_info(a)
      omm.model.graph.output.append(vi)
    elif type(a) == onnx.NodeProto:
      for o in a.output:
        vi = onnx.helper.make_empty_tensor_value_info(o)
        omm.model.graph.output.append(vi)
    else:
      raise Exception


'''


def _gen_op_maker(schema):
  onnx_op = schema.name
  inputs_args = [
      i.name if idx < schema.min_input else f"{i.name}=None"
      for idx, i in enumerate(schema.inputs)
  ]
  inputs_forloop = [i.name for i in schema.inputs]
  if len(schema.inputs) == 1:
    inputs_forloop.append("")
  if len(schema.inputs) != 0:
    inputs_args.append("")

  outputs_str = f'[f"{TENSOR_PREFIX}{onnx_op}_{{idx}}"]'
  if schema.name == "Split":
    if schema.since_version == 13:
      outputs_str = f'[f"{TENSOR_PREFIX}{onnx_op}_{{idx}}_{{i}}" for i in range(len(split))]'
    else:
      outputs_str = f'[f"{TENSOR_PREFIX}{onnx_op}_{{idx}}_{{i}}" for i in range(len(kwargs["split"]))]'

  return f'''@onnx_mm_export("v{schema.since_version}.{onnx_op}")
def {onnx_op}({', '.join(inputs_args)}**kwargs):
  _inputs = []
  for i in ({', '.join(inputs_forloop)}):
    _add_input(i, _inputs)

  idx = omm.op_counter[\"{onnx_op}\"]
  omm.op_counter[\"{onnx_op}\"] += 1
  node = onnx.helper.make_node(\"{onnx_op}\",
                               _inputs, {outputs_str},
                               name=f"{onnx_op}_{{idx}}",
                               **kwargs)
  onnx.checker.check_node(node, omm.ctx)
  omm.model.graph.node.append(node)
  return node
'''


def _gen_abs_op_maker(schema):
  onnx_op = schema.name
  return f'''def {onnx_op}(*args, **kwargs):
  schema = onnx.defs.get_schema("{onnx_op}",
                                max_inclusive_version=OPSET_VER,
                                domain="")
  return getattr(sys.modules[f"{{mod_name}}.ops"], 
                 f"v{{schema.since_version}}.{onnx_op}")(*args, **kwargs)
'''


def gen(output_dir=None, overwrite=False):
  if overwrite:
    shutil.rmtree(output_dir)
    os.makedirs(output_dir)
  if not os.path.exists(output_dir):
    os.makedirs(output_dir)
  abs_op_contents = {}
  file_contents = collections.defaultdict(list)
  all_schemas = onnx.defs.get_all_schemas_with_history()
  for schema in all_schemas:
    since_version = schema.since_version
    if str(since_version) not in file_contents:
      file_contents[str(since_version)].append(HEADER)
    if schema.name not in abs_op_contents:
      abs_op_contents[schema.name] = _gen_abs_op_maker(schema)
    file_contents[str(since_version)].append(_gen_op_maker(schema))
  for v, c in file_contents.items():
    with open(os.path.join(output_dir, f"op_ver_{v}.py"), "w") as f:
      f.write('''

'''.join(c))
  with open(os.path.join(output_dir, "__init__.py"), "w") as f:
    f.write(INIT_PY)
    f.write('''

'''.join([abs_op_contents[key] for key in sorted(abs_op_contents.keys())]))
    all_str = ', '.join([f'"{key}"' for key in sorted(abs_op_contents.keys())])
    f.write(f'''

__all__ = [\"Input\", \"Output\", {all_str}]''')
  with open(os.path.join(output_dir, "op_helper.py"), "w") as f:
    f.write(OP_HELPER_PY)


gen("./ops")



class CodeGenTemplate:

  @classmethod
  def autogen_head(cls):
    return '''# Autogenerated by onnx-pytorch.
'''

  @classmethod
  def imports(cls):
    return '''import glob
import os

import numpy as np
import torch
import torch.nn as nn
'''

  @classmethod
  def model(cls, model_init, model_forward, test_run_model):
    return f'''{cls.autogen_head()}
{cls.imports()}


class Model(nn.Module):
  def __init__(self):
    super(Model, self).__init__()
    self.__variables = {{
        os.path.basename(b)[:-4]: torch.from_numpy(np.load(b))
        for b in glob.glob(
            os.path.join(os.path.dirname(__file__), "variables", "*.npy"))
        if os.path.isfile(b)
    }}
    {model_init}

  def forward(self, *inputs):
    {model_forward}


{test_run_model}
'''



from tempfile import TemporaryDirectory

import os
import importlib.util

import numpy as np
import onnx
import onnxruntime
from onnxruntime.tools.symbolic_shape_infer import SymbolicShapeInference
import pytest
import torch

from onnx_model_maker import *
from onnx_model_maker.ops import *
from onnx_pytorch import code_gen

torch.set_printoptions(8)


class TestModel:

  def _run(self, inputs_np, onnx_model):
    model = onnx.ModelProto()
    model.CopyFrom(onnx_model)
    sess_options = onnxruntime.SessionOptions()
    session = onnxruntime.InferenceSession(model.SerializeToString(),
                                           sess_options)
    ort_outputs = session.run(None, {k: v for k, v in inputs_np})
    model.graph.ClearField("value_info")
    model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
    with TemporaryDirectory() as tmpdir:
      code_gen.gen(model, output_dir=tmpdir)
      spec = importlib.util.spec_from_file_location(
          "model", os.path.join(tmpdir, "model.py"))
      mod = importlib.util.module_from_spec(spec)
      spec.loader.exec_module(mod)
      pt_outputs = mod.test_run_model(
          [torch.from_numpy(v) for _, v in inputs_np])
      assert np.allclose(ort_outputs, [o.detach().numpy() for o in pt_outputs],
                         atol=1e-5,
                         rtol=1e-5,
                         equal_nan=True)

  def test_ugmarketing(self):
    ugmarkering_model_path = "/Users/wenhao/Projects/ai_speed/saved_model/model.onnx"
    if not os.path.exists(ugmarkering_model_path):
      return False
    model = onnx.load(ugmarkering_model_path)
    self._run(
        [("import/whole_input:0", np.random.randn(1, 2905).astype(np.float32))],
        model)


if __name__ == '__main__':
  pytest.main(['-s', 'test_onnx_model_zoo.py'])



import onnx
import torch

from onnx_pytorch.op_code_generators import OpCodeGenerator


class ReluOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(ReluOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)
    nn_name = "ReLU"
    init_str, forward_str = [], []
    init_str.append(f"self.{node.name} = nn.{nn_name}()")
    forward_str.append(f"{outputs_str[0]} = self.{node.name}({inputs_str[0]})")
    return {"init": init_str, "forward": forward_str}



from tempfile import TemporaryDirectory

import os
import importlib.util

import numpy as np
import onnx
import onnxruntime
from onnxruntime.tools.symbolic_shape_infer import SymbolicShapeInference
import pytest
import torch

from onnx_model_maker import *
from onnx_model_maker.ops import *
from onnx_pytorch import code_gen

torch.set_printoptions(8)


class TestBase:

  def _run(self, inputs_np):
    model = onnx.ModelProto()
    model.CopyFrom(omm.model)
    sess_options = onnxruntime.SessionOptions()
    session = onnxruntime.InferenceSession(model.SerializeToString(),
                                           sess_options)
    ort_outputs = session.run(None, {k: v for k, v in inputs_np})
    model.graph.ClearField("value_info")
    model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
    with TemporaryDirectory() as tmpdir:
      code_gen.gen(model, output_dir=tmpdir)
      spec = importlib.util.spec_from_file_location(
          "model", os.path.join(tmpdir, "model.py"))
      mod = importlib.util.module_from_spec(spec)
      spec.loader.exec_module(mod)
      pt_outputs = mod.test_run_model(
          [torch.from_numpy(v) for _, v in inputs_np])
      if type(pt_outputs) == torch.Tensor:
        pt_outputs = [pt_outputs.detach().numpy()]
      elif type(pt_outputs) in (list, tuple):
        pt_outputs = [o.detach().numpy() for o in pt_outputs]
      for l, r in zip(ort_outputs, pt_outputs):
        assert np.allclose(l, r, atol=1e-4, rtol=1e-4, equal_nan=True)

  def test_conv_flatten_relu(self):
    reset_model()
    nps = [np.random.randn(1, 3, 224, 224).astype(np.float32)]
    inputs = Input(*nps)
    conv_node = Conv(inputs[0],
                     np.random.randn(32, 3, 3, 3).astype(np.float32),
                     np.random.randn(32).astype(np.float32))
    flatten_node = Flatten(conv_node)
    relu_node = Relu(flatten_node)
    Output(relu_node)
    self._run(list(zip(inputs, nps)))

  def test_conv_batchnorm_maxpool_flatten_add_relu(self):
    reset_model()
    nps = [np.random.randn(1, 3, 224, 224).astype(np.float32)]
    inputs = Input(*nps)
    conv_node = Conv(inputs[0],
                     np.random.randn(32, 3, 3, 3).astype(np.float32),
                     np.random.randn(32).astype(np.float32))
    bn_node = BatchNormalization(
        conv_node,
        np.ones(32,).astype(np.float32),
        np.zeros(32,).astype(np.float32),
        np.random.randn(32).astype(np.float32),
        np.random.randn(32).astype(np.float32),
    )
    max_pool_node = MaxPool(bn_node,
                            kernel_shape=(3, 3),
                            strides=(2, 2),
                            pads=(0, 0, 1, 1))
    flatten_node = Flatten(max_pool_node, axis=1)
    add_node = Add(flatten_node, np.random.randn(1).astype(np.float32))
    relu_node = Relu(add_node)
    Output(relu_node)
    self._run(list(zip(inputs, nps)))

  def test_reshape(self):
    reset_model()
    nps = [np.random.randn(4,).astype(np.float32)]
    inputs = Input(*nps)
    Output(Reshape(inputs[0], np.array((2, 2)).astype(np.int64)))
    self._run(list(zip(inputs, nps)))

  def test_maxpool(self):
    reset_model()
    nps = [np.random.randn(1, 1, 5, 5).astype(np.float32)]
    inputs = Input(*nps)
    Output(MaxPool(inputs[0], kernel_shape=(3, 3), pads=(0, 0, 1, 1)))
    self._run(list(zip(inputs, nps)))

  def test_sigmoid(self):
    reset_model()
    nps = [np.random.randn(1, 10).astype(np.float32)]
    inputs = Input(*nps)
    Output(Sigmoid(inputs[0]))
    self._run(list(zip(inputs, nps)))

  def test_slice(self):
    reset_model()
    nps = [np.random.randn(5, 5).astype(np.float32)]
    inputs = Input(*nps)
    Output(
        Slice(inputs[0],
              np.array((2, 3)).astype(np.int64),
              np.array((4, 5)).astype(np.int64)))
    self._run(list(zip(inputs, nps)))

  def test_reduce_sum(self):
    reset_model()
    nps = [np.random.randn(1, 2, 3).astype(np.float32)]
    inputs = Input(*nps)
    Output(ReduceSum(inputs[0], np.array((1, 2)).astype(np.int64)))
    self._run(list(zip(inputs, nps)))

  def test_mul(self):
    reset_model()
    nps = [
        np.random.randn(1, 2, 3).astype(np.float32),
        np.random.randn(1, 2, 3).astype(np.float32)
    ]
    inputs = Input(*nps)
    Output(Mul(*inputs))
    self._run(list(zip(inputs, nps)))

  def test_mat_mul(self):
    reset_model()
    nps = [
        np.random.randn(5, 2, 3).astype(np.float32),
        np.random.randn(5, 3, 2).astype(np.float32)
    ]
    inputs = Input(*nps)
    Output(MatMul(*inputs))
    self._run(list(zip(inputs, nps)))

  def test_softmax(self):
    reset_model()
    nps = [np.random.randn(1, 10).astype(np.float32)]
    inputs = Input(*nps)
    Output(Softmax(inputs[0]))
    self._run(list(zip(inputs, nps)))

  def test_reciprocal(self):
    reset_model()
    nps = [np.random.randn(1, 10).astype(np.float32)]
    inputs = Input(*nps)
    Output(Reciprocal(inputs[0]))
    self._run(list(zip(inputs, nps)))

  def test_sqrt(self):
    reset_model()
    nps = [np.random.randn(1, 10).astype(np.float32)]
    inputs = Input(*nps)
    Output(Sqrt(inputs[0]))
    self._run(list(zip(inputs, nps)))

  def test_max(self):
    reset_model()
    nps = [
        np.random.randn(1, 10).astype(np.float32),
        np.random.randn(2, 10).astype(np.float32)
    ]
    inputs = Input(*nps)
    Output(Max(inputs))
    self._run(list(zip(inputs, nps)))

  def test_concat(self):
    reset_model()
    nps = [
        np.random.randn(1, 10).astype(np.float32),
        np.random.randn(2, 10).astype(np.float32)
    ]
    inputs = Input(*nps)
    Output(Concat(inputs, axis=0))
    self._run(list(zip(inputs, nps)))

  def test_split(self):
    reset_model()
    nps = [
        np.random.randn(1, 10).astype(np.float32),
    ]
    inputs = Input(*nps)
    Output(Split(inputs, split=np.array([2, 8]), axis=1))
    self._run(list(zip(inputs, nps)))


if __name__ == '__main__':
  pytest.main(['-s', 'test_base.py'])



import onnx
import torch
from onnx.numpy_helper import to_array

from onnx_pytorch.op_code_generators import OpCodeGenerator


class SliceOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(SliceOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    attr_value_dict = self.get_attr_value_dict(node)
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)
    init_str, forward_str = [], []
    d = len(value_infos[node.input[0]].type.tensor_type.shape.dim)
    starts, ends, axes, steps = self._get_starts_ends_axes_steps(
        attr_value_dict, d, node, initializers)
    slice_str = []
    for i in range(d):
      if i in axes:
        j = axes.index(i)
        s = [
            str(starts[j]) if starts[j] != 0 else "",
            str(ends[j]) if ends[j] < 2**31 else ""
        ]
        if steps[j] != 1:
          s.append(str(steps[j]))
        slice_str.append(":".join(s))
      else:
        slice_str.append(":")

    forward_str.append(
        f"{outputs_str[0]} = {inputs_str[0]}[{', '.join(slice_str)}]")
    return {"init": init_str, "forward": forward_str}

  def _get_starts_ends_axes_steps(self, attr_value_dict, d, node, initializers):
    axes = list(range(d))
    steps = [1] * len(axes)
    if self.onnx_ver > 1 and len(node.input) > 1:
      starts = initializers.get(node.input[1], None)
      ends = initializers.get(node.input[2], None)
      starts = to_array(starts)
      ends = to_array(ends)
      if len(node.input) > 3:
        axes = initializers.get(node.input[3], None)
      if len(node.input) > 4:
        steps = initializers.get(node.input[4], None)
      assert starts is not None or ends is not None or axes is not None or steps is not None, "Currently SliceOpCodeGenerator only support all of [starts, ends, axes, steps] is in initializers."
      if len(node.input) > 3:
        axes = to_array(axes)
      if len(node.input) > 4:
        steps = to_array(steps)
    else:
      starts = attr_value_dict["starts"]
      ends = attr_value_dict["ends"]
      axes = attr_value_dict.get("axes", axes)
    return starts, ends, axes, steps



import onnx
import torch

from onnx_pytorch.op_code_generators import OpCodeGenerator


class GemmOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(GemmOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    attr_value_dict = self.get_attr_value_dict(node)
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)

    if attr_value_dict["transA"] == 1:
      inputs_str[0] = f"torch.transpose({inputs_str[0]}, 0, 1)"
    if attr_value_dict["transB"] == 1:
      inputs_str[1] = f"torch.transpose({inputs_str[1]}, 0, 1)"

    init_str, forward_str = [], []
    forward_str.append(
        f"{outputs_str[0]} = {attr_value_dict['alpha']} * torch.matmul({', '.join(inputs_str[:2])}) + {attr_value_dict['beta']} * {inputs_str[2]}"
    )

    return {"init": init_str, "forward": forward_str}



import onnx
import torch

from onnx_pytorch.op_code_generators import OpCodeGenerator


class GlobalAveragePoolOpCodeGenerator(OpCodeGenerator):

  def __init__(self,
               onnx_ver=onnx.defs.onnx_opset_version(),
               torch_ver=torch.__version__):
    super(GlobalAveragePoolOpCodeGenerator, self).__init__(onnx_ver, torch_ver)

  def gen(self, node, value_infos, initializers):
    inputs_str, outputs_str = self.gen_input_output_string(node, initializers)
    d = len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2
    params_str = self.gen_params_str(
        kernel_size=f"self.{node.input[0]}.shape[{d}:]")

    nn_name = f"functional.avg_pool{d}d"
    init_str, forward_str = [], []
    init_str.append(f"self.{node.name} = nn.{nn_name}")
    forward_str.append(
        f"{outputs_str[0]} = self.{node.name}({inputs_str[0]}, **{{{params_str}}})"
    )

    return {"init": init_str, "forward": forward_str}
